# authentic2 - versatile identity manager
# Copyright (C) 2010-2021 Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from django.db import connection, transaction


def safe_get_or_create(model, defaults=None, **kwargs):
    assert (
        getattr(settings, 'TESTING', False) or not connection.in_atomic_block
    ), 'safe_get_or_create cannot be used in inside a transaction'

    try:
        return model.objects.get(**kwargs), False
    except model.DoesNotExist:
        pass
    with transaction.atomic():
        with connection.cursor() as cur:
            cur.execute('LOCK TABLE "%s" IN EXCLUSIVE MODE' % model._meta.db_table)
        return model.objects.get_or_create(defaults=defaults, **kwargs)
